package info.silin.an10na.bind;

import java.util.HashMap;
import java.util.Map;

/**
 * Registeres IDataBindings under String keys, makes individual and bulk updates and binding queries
 * per id available.
 *
 * @author kostja
 */
public class BindingRegistry {

    private Map<String, IDataBinding<?, ?>> bindings = new HashMap<String, IDataBinding<?, ?>>();

    /**
     * Retrieve the map off all registered bindings.
     *
     * @return map of registered bindings
     */
    public Map<String, IDataBinding<?, ?>> getBindings() {
        return bindings;
    }

    public void setBindings(final Map<String, IDataBinding<?, ?>> bindings) {
        this.bindings = bindings;
    }

    /**
     * Make a bindign available under the id to clients of the BindingRegistry.
     *
     * @param id      the id of the binding.
     * @param binding the previous binding for the id or null if no binding was registered under the
     *                id.
     * @return returns the binding that was registered under the same id or null if no binding
     * has been registered under the same id.
     */
    public IDataBinding<?, ?> register(final String id, final IDataBinding<?, ?> binding) {
        return bindings.put(id, binding);
    }

    /**
     * Retrieves a registered binding by id.
     *
     * @param id identifier of the binding.
     * @return the binding registered under the id or null if no binding  was registered under the
     *         id.
     */
    public IDataBinding<?, ?> get(final String id) {
        return bindings.get(id);
    }

    /**
     * Check if a binding is registered under an id.
     *
     * @param id the identifier fo the binding
     * @return true if a bindign was registered with this id, false otherwise.
     */
    public boolean isBound(final String id) {
        return bindings.containsKey(id);
    }

    /**
     * Check if a binding was registered.
     *
     * @param binding the binding to look for.
     * @return true if the binding was registered, false otherwise
     */
    public boolean isBound(final IDataBinding<?, ?> binding) {
        return bindings.containsValue(binding);
    }

    //TODO : return all bindings for a source/target

    /**
     * Update a binding registered with the id.
     *
     * @param id id of the binding to be updated
     * @return True if a binding was found and updated, false otherwise.
     */
    public boolean update(final String id){

        final IDataBinding<?, ?> binding = bindings.get(id);
        if(binding != null){
            binding.update();
            return true;
        }
        return false;
    }

    /**
     * Update all bindings.
     */
    public void update() {
        for (final IDataBinding<?, ?> binding : bindings.values()) {
            binding.update();
        }
    }
}
